@@ -1,6 +1,6 @@
 # -*- coding: latin-1 -*-
 #
-# Copyright 2011-2020 Ghent University
+# Copyright 2011-2021 Ghent University
 #
 # This file is part of vsc-install,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -48,6 +48,7 @@ import shutil
 import re
 
 import setuptools
+import setuptools.dist
 import setuptools.command.test
 
 from distutils import log  # also for setuptools
@@ -75,7 +76,7 @@ except ImportError:
 # Test that these are matched by a .gitignore pattern
 GITIGNORE_PATTERNS = ['.pyc', '.pyo', '~']
 # .gitnore needs to contain these exactly
-GITIGNORE_EXACT_PATTERNS = ['.eggs']
+GITIGNORE_EXACT_PATTERNS = ['.eggs*']
 
 # private class variables to communicate
 # between VscScanningLoader and VscTestCommand
@@ -130,7 +131,7 @@ if log.Log.__name__ != 'NewLog':
 
 # available authors
 ag = ('Andy Georges', 'andy.georges@ugent.be')
-asg = ('Álvaro Simón García', 'alvaro.simongarcia@UGent.be')
+asg = ('Ãlvaro SimÃ³n GarcÃ­a', 'alvaro.simongarcia@UGent.be')
 eh = ('Ewan Higgs', 'Ewan.Higgs@UGent.be')
 jt = ('Jens Timmerman', 'jens.timmerman@ugent.be')
 kh = ('Kenneth Hoste', 'kenneth.hoste@ugent.be')
@@ -143,6 +144,12 @@ sm = ('Samuel Moors', 'samuel.moors@vub.be')
 bh = ('Balazs Hajgato', 'Balazs.Hajgato@UGent.be')
 ad = ('Alex Domingo', 'alex.domingo.toro@vub.be')
 
+# available remotes
+GIT_REMOTES = [
+    'hpcugent',
+    'sisc-hpc',
+]
+
 # Regexp used to remove suffixes from scripts when installing(/packaging)
 REGEXP_REMOVE_SUFFIX = re.compile(r'(\.(?:py|sh|pl))$')
 
@@ -159,7 +166,7 @@ URL_GHUGENT_HPCUGENT = 'https://github.ugent.be/hpcugent/%(name)s'
 
 RELOAD_VSC_MODS = False
 
-VERSION = '0.15.5'
+VERSION = '0.17.5'
 
 log.info('This is (based on) vsc.install.shared_setup %s' % VERSION)
 log.info('(using setuptools version %s located at %s)' % (setuptools.__version__, setuptools.__file__))
@@ -172,6 +179,7 @@ NO_PREFIX_PYTHON_BDIST_RPM = ['pbs_python']
 # Hardcode map of python dependency prefix to their rpm python- flavour prefix
 PYTHON_BDIST_RPM_PREFIX_MAP = {
     'pycrypto': 'python%s-crypto',
+    'psycopg2': 'python%s-psycopg2',
 }
 
 SHEBANG_BIN_BASH = "#!/bin/bash"
@@ -220,6 +228,9 @@ KNOWN_LICENSES = {
 # a whitelist of licenses that allow pushing to pypi during vsc_release
 PYPI_LICENSES = ['LGPLv2+', 'GPLv2']
 
+# environment variable name to set when building rpms from vsc-install managed repos
+#    indicates the python version it is being build for
+VSC_RPM_PYTHON = 'VSC_RPM_PYTHON'
 
 def _fvs(msg=None):
     """
@@ -260,6 +271,46 @@ def _fvs(msg=None):
     return klass
 
 
+def _read(source, read_lines=False):
+    """read a file, either in full or as a list (read_lines=True)"""
+    with open(source) as file_handle:
+        if read_lines:
+            txt = file_handle.readlines()
+        else:
+            txt = file_handle.read()
+    return txt
+
+
+# for sufficiently recent version of setuptools, we can hijack the 'get_egg_cache_dir' method
+# to control the .eggs directory being used
+if hasattr(setuptools.dist.Distribution, 'get_egg_cache_dir'):
+    setuptools.dist.Distribution._orig_get_egg_cache_dir = setuptools.dist.Distribution.get_egg_cache_dir
+
+    # monkey patch setuptools to use different .eggs directory depending on Python version being used
+    def get_egg_cache_dir_pyver(self):
+        egg_cache_dir = self._orig_get_egg_cache_dir()
+
+        # the original get_egg_cache_dir creates the .eggs directory if it doesn't exist yet,
+        # but we want to have it versioned, so we rename it
+        egg_cache_dir_pyver = egg_cache_dir + '.py%s' % sys.version_info[0]
+        try:
+            if not os.path.exists(egg_cache_dir_pyver):
+                os.rename(egg_cache_dir, egg_cache_dir_pyver)
+        except OSError as err:
+            raise OSError("Failed to rename %s to %s: %s" % (egg_cache_dir, egg_cache_dir_pyver, err))
+
+        return egg_cache_dir_pyver
+
+    setuptools.dist.Distribution.get_egg_cache_dir = get_egg_cache_dir_pyver
+
+# for ancient setuptools version (< 7.0), get_egg_cache_dir is not there yet
+# in that case, just hard remove the existing .eggs directory, to force re-creating it
+else:
+    eggs_dir = os.path.join(os.getcwd(), '.eggs')
+    if os.path.exists(eggs_dir):
+        shutil.rmtree(eggs_dir)
+
+
 class vsc_setup(object):
     """
     Store these Constants in a separate class instead of creating them at runtime,
@@ -315,13 +366,14 @@ class vsc_setup(object):
         elif not os.path.isfile(filename):
             raise Exception('cannot find file %s to get name from' % filename)
 
-        txt = open(filename).read()
+        txt = _read(filename)
 
         # First ones are from PKG-INFO
         # second one is .git/config
 
         # multiline search
         # github pattern for hpcugent, not fork
+        github_domain_pattern = '(?:%s)' % '|'.join(GIT_REMOTES)
         all_patterns = {
             'name': [
                 r'^Name:\s*(.*?)\s*$',
@@ -329,8 +381,8 @@ class vsc_setup(object):
             ],
             'url': [
                 r'^Home-page:\s*(.*?)\s*$',
-                r'^\s*url\s*=\s*((?:https?|ssh).*?github.*?[:/](?:hpcugent|sisc-hpc)/.*?)\.git\s*$',
-                r'^\s*url\s*=\s*(git[:@].*?github.*?[:/](?:hpcugent|sisc-hpc)/.*?)(?:\.git)?\s*$',
+                r'^\s*url\s*=\s*((?:https?|ssh).*?github.*?[:/]%s/.*?)(?:\.git)?\s*$' % github_domain_pattern,
+                r'^\s*url\s*=\s*(git[:@].*?github.*?[:/]%s/.*?)(?:\.git)?\s*$' % github_domain_pattern,
             ],
             'download_url': [
                 r'^Download-URL:\s*(.*?)\s*$',
@@ -354,7 +406,8 @@ class vsc_setup(object):
             self.private_repo = True
 
         if 'url' not in res:
-            raise KeyError("Missing url in git config %s. (Missing mandatory hpcugent or sisc-hpc remote?)" % (res))
+            allowed_remotes = ', '.join(GIT_REMOTES)
+            raise KeyError("Missing url in git config %s. (Missing mandatory remote? %s)" % (res, allowed_remotes))
 
         # handle git://server/user/project
         reg = re.search(r'^(git|ssh)://', res.get('url', ''))
@@ -398,7 +451,8 @@ class vsc_setup(object):
         # primitive gitignore
         gitignore = os.path.join(base_dir, '.gitignore')
         if os.path.isfile(gitignore):
-            all_patterns = [l for l in [l.strip() for l in open(gitignore).readlines()] if l and not l.startswith('#')]
+            all_patterns = [l for l in [l.strip() for l in _read(gitignore, read_lines=True)]
+                            if l and not l.startswith('#')]
 
             patterns = [l.replace('*', '.*') for l in all_patterns if l.startswith('*')]
             reg = re.compile('^('+'|'.join(patterns)+')$')
@@ -440,7 +494,7 @@ class vsc_setup(object):
             if '__init__.py' in files or package in excluded_pkgs:
                 # Force vsc shared packages/namespace
                 if '__init__.py' in files and (package == 'vsc' or package.startswith('vsc.')):
-                    init = open(os.path.join(root, '__init__.py')).read()
+                    init = _read(os.path.join(root, '__init__.py'))
                     if not re.search(r'^import\s+pkg_resources\n{1,3}pkg_resources.declare_namespace\(__name__\)$',
                                      init, re.M):
                         raise Exception(('vsc namespace packages do not allow non-shared namespace in dir %s.'
@@ -509,9 +563,7 @@ class vsc_setup(object):
                 os.unlink(dest)
                 self.copy_file(os.path.join(self.setup.REPO_BASE_DIR, *paths), dest)
 
-            fh = open(dest, 'r')
-            code = fh.read()
-            fh.close()
+            code = _read(dest)
 
             return dest, code
 
@@ -566,9 +618,7 @@ class vsc_setup(object):
                 raise Exception("sdist requires access shared_setup source (%s)" % err)
 
             try:
-                fh = open(dest, 'w')
-                fh.write(source_code)
-                fh.close()
+                self._write(dest, source_code)
             except IOError as err:
                 raise IOError("Failed to write NEW_SHARED_SETUP source to %s (%s)" % (dest, err))
 
@@ -619,7 +669,7 @@ class vsc_setup(object):
                 pyshebang_reg = re.compile(r'\A%s.*$' % SHEBANG_ENV_PYTHON, re.M)
                 for fn in scripts:
                     # includes newline
-                    first_line = open(os.path.join(base_dir, fn)).readline()
+                    first_line = _read(os.path.join(base_dir, fn), read_lines=True)[0]
                     if pyshebang_reg.search(first_line):
                         log.info("going to adapt shebang for script %s" % fn)
                         dest, code = self._recopy(base_dir, fn)
@@ -1199,12 +1249,18 @@ class vsc_setup(object):
                 and name does not start with python-
         """
 
+        def fix_range(txt):
+            """Convert , separated version requirements in explicit repeated versions"""
+            parts = txt.split(',')
+            first = parts.pop(0)
+            prog = first.split(' ')[0]
+            return ", ".join([first]+["%s %s" % (prog, x.strip()) for x in parts])
+
         if isinstance(name, (list, tuple)):
             klass = _fvs('sanitize')
-            return ",".join([klass.sanitize(r) for r in name])
-
+            return "\n    ".join([klass.sanitize(r) for r in name])
         else:
-            pyversuff = os.environ.get('VSC_RPM_PYTHON', None)
+            pyversuff = os.environ.get(VSC_RPM_PYTHON, None)
             if pyversuff in ("1", "2", "3"):
                 # enable VSC-style naming for Python packages: use 'python2-*' or 'python3-*',
                 # unless '1' is used as value for $VSC_RPM_PYTHON, then use 'python-*' for legacy behaviour
@@ -1214,7 +1270,7 @@ class vsc_setup(object):
                 # hardcoded prefix map
                 for pydep, rpmname in PYTHON_BDIST_RPM_PREFIX_MAP.items():
                     if name.startswith(pydep):
-                        newname = (rpmname+name[len(pydep):]) % pyversuff
+                        newname = fix_range((rpmname+name[len(pydep):]) % pyversuff)
                         log.debug("new sanitized name %s from map (old %s)", newname, name)
                         return newname
 
@@ -1224,11 +1280,11 @@ class vsc_setup(object):
                                  or name.startswith('vsc'))
 
                 if is_python_pkg:
-                    newname = 'python%s-%s' % (pyversuff, name)
+                    newname = fix_range('python%s-%s' % (pyversuff, name))
                     log.debug("new sanitized name %s (old %s)", newname, name)
                     return newname
 
-            return name
+            return fix_range(name)
 
 
 
@@ -1310,6 +1366,11 @@ class vsc_setup(object):
         new_target['license'] = lic_name
         classifiers.append(lic_classifier)
 
+        # a dict with key the new_target key to search and replace
+        #    value is a list of 2-element (pattern, replace) lists passed to re.sub
+        #    if returning value is empty, it is not added after the replacement
+        vsc_filter_rpm = target.pop('vsc_filter_rpm', {})
+
         # set name, url, download_url (skip name if it was specified)
         update = self.get_name_url(version=target['version'], license_name=lic_name)
         if 'name' in target:
@@ -1333,7 +1394,7 @@ class vsc_setup(object):
             if 'long_description' in target:
                 log.info(('Going to ignore the provided long_descripton.'
                           'Set it in the %s or disable vsc_description') % README)
-            readmetxt = open(readme).read()
+            readmetxt = _read(readme)
 
             # look for description block, read text until double empty line or new block
             # allow 'words with === on next line' or comment-like block '# title'
@@ -1406,19 +1467,35 @@ class vsc_setup(object):
                     new_target[k] = type(v)()
                     new_target[k] += v
 
+        tests_requires = new_target.setdefault('tests_require', [])
         if sys.version_info < (2, 7):
             # py26 support dropped in 0.8, and the old versions don't detect enough
             log.info('no prospector support in py26 (or older)')
-            tests_requires = new_target.setdefault('tests_require', [])
-            new_target['tests_require'] = [x for x in tests_requires if 'prospector' not in x]
-        else:
+            tests_requires = [x for x in tests_requires if 'prospector' not in x]
+        elif sys.version_info < (3, 0):
             log.info('adding prospector to tests_require')
-            tests_requires = new_target.setdefault('tests_require', [])
+            # limit version to avoid pulling in python 3.x only versions
+            tests_requires.append('zipp < 2.0')
+            tests_requires.append('configparser < 5.0')
             # Python 2.x support was removed in pydocstyle 4.0, so stick to latest release before 4.0
             tests_requires.append('pydocstyle < 4.0')
             # fix from https://github.com/PyCQA/prospector/pull/323 required to avoid infinite recursion
-            tests_requires.append('prospector >= 1.1.6.4')
-            new_target['tests_require'] = tests_requires
+            tests_requires.append('prospector >= 1.1.6.4, < 1.3')
+            # mock 4.x is no longer compatible with Python 2
+            tests_requires.append('mock < 4.0')
+            # isort 5.0 is no longer compatible with Python 2
+            tests_requires.append('isort < 5.0')
+        else:
+            tests_requires.extend([
+                # stick to older flake8, to avoid version conflict on pyflakes dependency
+                # flake8 3.9.0 requires pyflakes<2.4.0,>=2.3.0
+                # prospector 1.3.1 requires pyflakes<2.3.0,>=2.2.0
+                'flake8 < 3.9.0',
+                'prospector',
+                'mock',
+            ])
+
+        new_target['tests_require'] = tests_requires
 
         if self.private_repo:
             urls = [
@@ -1448,8 +1525,30 @@ class vsc_setup(object):
                     new_target['dependency_links'] += [''.join([git_scheme, url, '/hpcugent/', dep_name, '.git#egg=',
                                                                 dep_name_version])]
 
+        if VSC_RPM_PYTHON in os.environ:
+            for key, pattern_replace_list in vsc_filter_rpm.items():
+                def search_replace(txt):
+                    for pattern, replace in pattern_replace_list:
+                        txt = re.sub(pattern, replace, txt)
+                    return txt
+
+                if key in new_target:
+                    log.debug("Found VSC_RPM_PYTHON set and vsc_filter_rpm for %s set to %s", key, pattern_replace_list)
+                    old = new_target.pop(key)
+                    if isinstance(old, list):
+                        # remove empty strings
+                        new = [y for y in [search_replace(x) for x in old] if y]
+                    else:
+                        log.error("vsc_filter_rpm does not support %s for %s", type(old), key)
+                        sys.exit(1)
+                    if new:
+                        log.debug("new vsc_filter_rpm value for %s: %s", key, new)
+                        new_target[key] = new
+                    else:
+                        log.debug("new vsc_filter_rpm value for %s was empty, not adding it back", key)
+
         log.debug("New target = %s" % (new_target))
-        print(new_target)
+        print("new target", new_target)
         return new_target
 
     @staticmethod
@@ -1502,7 +1601,6 @@ class vsc_setup(object):
             exclude files provided by packages that are shared
                 excluded_pkgs_rpm: is a list of packages, default to ['vsc']
                 set it to None when defining own function
-            generate the setup.cfg using build_setup_cfg_for_bdist_rpm
         """
         pkgs = target.pop('excluded_pkgs_rpm', ['vsc'])
         if pkgs is not None:
@@ -1513,7 +1611,6 @@ class vsc_setup(object):
         # therefor we regenerate self.package files with the excluded pkgs as extra param
         self.package_files = self.files_in_packages(excluded_pkgs=pkgs)
         _fvs('prepare_rpm').SHARED_TARGET['packages'] = self.generate_packages()
-        self.build_setup_cfg_for_bdist_rpm(target)
 
     def action_target(self, target, setupfn=None, extra_sdist=None, urltemplate=None):
         """
@@ -1544,8 +1641,12 @@ class vsc_setup(object):
             self.cleanup()
 
         self.prepare_rpm(target)
-        x = self.parse_target(target, urltemplate)
-        setupfn(**x)
+
+        new_target = self.parse_target(target, urltemplate)
+        # generate the setup.cfg using build_setup_cfg_for_bdist_rpm
+        self.build_setup_cfg_for_bdist_rpm(new_target)
+
+        setupfn(**new_target)
 
 # here for backwards compatibility
 SHARED_TARGET = _fvs('SHARED_TARGET').SHARED_TARGET
